%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% At each stage of the network, we have a 3D layer of size W x H x D
% We say that this layer has width W, height H, depth D
%
% 'conv' and 'pool' layer transforms the previous layer to a new size, W' x H x D
% Normalization and ReLu Layers leave the previous layer size unchanged
%
% 
% Final layer must be a 'softmaxloss' layer.
% Before the final  'softmaxloss' layer, there must be at least one fully connected layer
% 
% Here is how to get a fully connected layer with q units
% Suppose your current layer has dimensions a x b x d
% Use convolutional layer with parameters [a,b,d] with a stride of '1' and
% no padding:
% 
%  struct('type', 'conv', 'filters', f*randn(a,b,d,q, 'single'), ... 
%                           'biases', zeros(1, q, 'single'), ...       
%                           'stride', 1,'pad', 0) ;                                                                                                      
%  If this fully connected layer connects to the 'softmaxloss' layer, then
% you want to make q = number of classes, which is 10 in our case
% If this fully connected layer connects to another fully connected layer,
% q can be arbitrary.  
% Example: say we want two fully connected layers before the final 'softmaxloss' layer.
% Suppose current layer has size 20 x 20 x 40. First create a fully
% connected layer, with, say 100 units:
% 
%  struct('type', 'conv', 'filters', f*randn(20,20,40,100, 'single'), ... 
%                           'biases', zeros(1, 100, 'single'), ...       
%                           'stride', 1,'pad', 0) ;
%
%  Now the current layer has size 1 x 1 x 100
%
%  Then create another connected layer with 10 units (= number of classes)                   
% 
%  struct('type', 'conv', 'filters', f*randn(1,1,100,10, 'single'), ... 
%                           'biases', zeros(1, 10, 'single'), ...       
%                           'stride', 1,'pad', 0) ;                                                                                                      
% 
% Now current layer has size 1 x 1 x 10, which is good for 'softmaxloss'
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

f=1/100 ;  %constant to make initial filter weights small
net.layers = {} ; % start with empty network
numClasses = 10;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% IMPORTANT:  
% The input image is assumed to be 200x200x3
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  FIRST STAGE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% add a convolutional layer
net.layers{end+1} = struct('type', 'conv', ...
                           'filters', f*randn(5,5,3,30, 'single'), ... % hight and width of output image is smaller by 4
                           'biases', zeros(1, 30, 'single'), ...       % 30 biases because number of filters is 30
                           'stride', 2, ...                            % makes image  twice smaller
                           'pad', 0) ;                                 % output has size 98x98x30                                                                      
                                                                       % 98x98 are width and height, 30 is number of
                                                                       % channels
                                                               
% add a spatial normalization layer                       
 net.layers{end+1} = struct('type', 'spnorm', ...              % Locally normalizes output of previous layer 
                             'param', [5 5 0.5 0.75]) ;         % in each chanel separately. 
                                                                % Normalizationwindow is [5 5], 0.5 and 0.75
                                                                % are parameters of normalizaiton
                                                          

                       
% add a ReLu layer
net.layers{end+1} = struct('type', 'relu') ;

% add a max pool  layer
net.layers{end+1} = struct('type', 'pool', ...
                           'method', 'max', ...
                           'pool', [3 3], ...  % pools info from 3 by 3 subarrays. Crops image by 1 in each direction
                           'stride', 2, ...    % makes image twice smaller
                           'pad', 0) ;         % output has size 48x48x30
                        
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% END OF FIRST STAGE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%                                             
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% SECOND STAGE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%                       

% add a convolutional  layer                       
net.layers{end+1} = struct('type', 'conv', ...
                           'filters', f*randn(5,5,30,30, 'single'),...  % hight and width of output image is smaller by 4
                           'biases', zeros(1,30,'single'), ...
                           'stride', 2, ...                             % makes image twice smaller
                           'pad', 0) ;                                  % output has size 22x22x30
                       

% add a spatial normalization layer                       
  net.layers{end+1} = struct('type', 'spnorm', ...
                             'param', [5 5 0.5 0.75]) ;
                        
% add a ReLu layer
  net.layers{end+1} = struct('type', 'relu') ;

% add a max pool  layer
net.layers{end+1} = struct('type', 'pool', ...
                           'method', 'max', ...
                           'pool', [3 3], ...  % pools info from 3 by 3 subarrays. Crops image by 1 in each direction
                           'stride', 2, ...    % makes image twice smaller
                           'pad', 0) ;         % output has size 10x10x30

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% END OF SECOND STAGE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%                       
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% FULL CONNECTED LAYERS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% fully connected layer, implemented as a convolutional layer
net.layers{end+1} = struct('type', 'conv', ...
                           'filters', f*randn(10,10,30,numClasses, 'single'),...
                           'biases', zeros(1,numClasses,'single'), ...
                           'stride', 1, ...
                           'pad', 0) ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% FINAL CLASSIFICATION LAYER %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% final "softmax" layer,performs classication                        
net.layers{end+1} = struct('type', 'softmaxloss'); 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    END OF NETWORK DESIGN    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
